<?php

/**
 * @file
 * Contains post update hooks for the module.
 */

declare(strict_types=1);

use Drupal\Core\Entity\Sql\SqlEntityStorageInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Modifies the weight column of entity hierarchy fields.
 */
function entity_hierarchy_post_update_update_weight_column(array &$sandbox): TranslatableMarkup {
  if (!isset($sandbox['fields'])) {
    // This must be the first run. Initialize the sandbox.
    $sandbox['fields'] = \Drupal::service('entity_field.manager')->getFieldMapByFieldType('entity_reference_hierarchy');
    $sandbox['max'] = count($sandbox['fields']);
  }
  $entityTypes = array_keys($sandbox['fields']);
  $entityType = array_shift($entityTypes);
  $fields = $sandbox['fields'][$entityType];
  unset($sandbox['fields'][$entityType]);
  _entity_hierarchy_update_field_weight_column($fields, $entityType);

  $sandbox['#finished'] = empty($sandbox['fields']) ? 1 : ($sandbox['max'] - count($sandbox['fields'])) / $sandbox['max'];
  if ($sandbox['#finished'] !== 1) {
    return new TranslatableMarkup('Updated fields for @types', [
      '@types' => \Drupal::entityTypeManager()->getDefinition($entityType)->getPluralLabel(),
    ]);
  }
  return new TranslatableMarkup('Updated all entity-hierarchy fields');
}

/**
 * Updates weight column for all fields for a given entity type.
 */
function _entity_hierarchy_update_field_weight_column(array $fields, string $entityTypeId): void {
  // The process is:
  // 1. Update the SQL table schema for all relevant tables.
  // 2. Inform Drupal of the expected table schemas.
  // 3. Update Drupal's entity type definitions.
  $schema = \Drupal::database()->schema();
  $entityTypeManager = \Drupal::entityTypeManager();
  $entityFieldManager = \Drupal::service('entity_field.manager');
  $entityStorageSchemaSql = \Drupal::keyValue('entity.storage_schema.sql');
  /** @var \Drupal\Core\Entity\EntityLastInstalledSchemaRepositoryInterface $lastInstalledSchemaRepository */
  $lastInstalledSchemaRepository = \Drupal::service('entity.last_installed_schema.repository');

  $entityStorage = $entityTypeManager->getStorage($entityTypeId);
  if (!$entityStorage instanceof SqlEntityStorageInterface) {
    return;
  }

  $entityType = $entityTypeManager->getDefinition($entityTypeId);
  // Loads definitions for all fields (even non $fieldType).
  $entityFieldStorageDefinitions = $entityFieldManager->getFieldStorageDefinitions($entityTypeId);
  /** @var \Drupal\Core\Entity\Sql\DefaultTableMapping $tableMapping */
  $tableMapping = $entityStorage->getTableMapping($entityFieldStorageDefinitions);
  // Intersect $fieldType fields with storage definitions for all fields.
  /** @var \Drupal\Core\Field\FieldStorageDefinitionInterface[] $fieldDefinitions */
  $fieldDefinitions = array_intersect_key($entityFieldStorageDefinitions, $fields);

  // Iterate over all $fieldType field definitions for this entity type.
  foreach ($fieldDefinitions as $fieldDefinition) {
    $fieldName = $fieldDefinition->getName();
    $tables = [];
    $tables[] = $tableMapping->getFieldTableName($fieldName);
    if ($entityType->isRevisionable() && $fieldDefinition->isRevisionable()) {
      $tables[] = $tableMapping->getDedicatedRevisionTableName($fieldDefinition);
    }

    // Field type column names map to real table column names.
    $columns = $tableMapping->getColumnNames($fieldName);
    $fieldColumnName = $columns['weight'];

    foreach ($tables as $table) {
      // Change the weight field spec.
      $schema->changeField($table, $fieldColumnName, $fieldColumnName, [
        'type' => 'int',
        'unsigned' => FALSE,
        'size' => 'small',
        'not null' => TRUE,
      ]);
    }

    // Update the tracked entity table schema.
    $schemaKey = "$entityTypeId.field_schema_data.$fieldName";
    $fieldSchemaData = $entityStorageSchemaSql->get($schemaKey);
    foreach ($fieldSchemaData as $tableName => $fieldSchema) {
      $fieldSchemaData[$tableName]['fields'][$fieldColumnName] = [
        'type' => 'int',
        'unsigned' => FALSE,
        'size' => 'small',
        'not null' => TRUE,
      ];
    }
    $entityStorageSchemaSql->set($schemaKey, $fieldSchemaData);

    // Update cached entity definitions.
    $definitions = $lastInstalledSchemaRepository->getLastInstalledFieldStorageDefinitions($entityTypeId);
    $definitions[$fieldName] = $fieldDefinition;
    $lastInstalledSchemaRepository->setLastInstalledFieldStorageDefinitions($entityTypeId, $definitions);
  }
}
